@@ -166,16 +166,6 @@ sub loadmod_Types {
   die "can't find PDL::Types: $foo and $@";
 }
 
-my %sym2type;
-sub load_Types {
-  loadmod_Types();
-  return \%sym2type if %sym2type;
-  %sym2type = map {
-    $PDL::Types::typehash{$_}->{'sym'} => $PDL::Types::typehash{$_}->{'ctype'}
-  } keys %PDL::Types::typehash;
-  \%sym2type;
-}
-
 my $inc = defined $PDL::Config{MALLOCDBG}->{include} ?
   "$PDL::Config{MALLOCDBG}->{include}" : '';
 my $libs = defined $PDL::Config{MALLOCDBG}->{libs} ?
@@ -264,7 +254,7 @@ sub isbigendian {
 
 # return exit code, so 0 = OK
 sub genpp {
-   my $sym2type = load_Types();
+   loadmod_Types();
    my $gotstart = 0; my @gencode = ();
    my ($loopvar, $indent);
 
@@ -294,7 +284,7 @@ sub genpp {
 
          push @gencode, $`;
 
-         print flushgeneric($indent, $loopvar, \@gencode, $sym2type);  # Output the generic code
+         print flushgeneric($indent, $loopvar, \@gencode);  # Output the generic code
 
          print $';  # End of genric code
          $gotstart = 0;
@@ -313,15 +303,12 @@ sub genpp {
 }
 
 sub flushgeneric {  # Construct the generic code switch
-   my ($indent, $loopvar, $gencode, $sym2type) = @_;
+   my ($indent, $loopvar, $gencode) = @_;
    my @m;
    push @m, $indent,"switch ($loopvar) {\n\n";
 
-   for my $case (PDL::Types::typesrtkeys()){
-
-     my $type = $sym2type->{$case};
-
-     my $ppsym = $PDL::Types::typehash{$case}->{ppsym};
+   for my $t (PDL::Types::types()) {
+     my ($case, $type, $ppsym) = map $t->$_, qw(sym ctype ppsym);
      push @m, $indent,"case $case:\n"; # Start of this case
      push @m, $indent,"   {";
 
@@ -381,39 +368,25 @@ EOF
 }
 
 # Expects list in format:
-# [gtest.pd, GTest, PDL::GTest,     ['../GIS/Proj', ...] ], [...]
-# source,    prefix,module/package, optional deps
+# [gtest.pd, GTest, PDL::GTest[, PDL::XSPkg] ], [...]
+# source,    prefix,module/package, optional pp_addxs destination
 # The idea is to support in future several packages in same dir - EUMM
 #   7.06 supports
-# each optional dep is a relative dir that a "make" will chdir to and
-# "make" first - so the *.pd file can then "use" what it makes
 
 # This is the function internal for PDL.
 
 sub pdlpp_postamble_int {
-	join '',map { my($src,$pref,$mod, $deps) = @$_;
-        die "If give dependencies, must be array-ref" if $deps and !ref $deps;
+	join '',map { my($src,$pref,$mod,$callpack) = @$_;
 	my $w = whereami_any();
 	$w =~ s%/((PDL)|(Basic))$%%;  # remove the trailing subdir
 	my $top = File::Spec->abs2rel($w);
 	my $basic = File::Spec->catdir($top, 'Basic');
 	my $core = File::Spec->catdir($basic, 'Core');
-	my $gen = File::Spec->catdir($basic, 'Gen');
-        my $depbuild = '';
-        for my $dep (@{$deps || []}) {
-            my $target = '';
-            if ($dep eq 'core') {
-                $dep = $top;
-                $target = ' core';
-            }
-            $dep =~ s#([\(\)])#\\$1#g; # in case of unbalanced (
-            $depbuild .= _oneliner("exit(!(chdir q($dep) && !system(q(\$(MAKE)$target))))");
-            $depbuild .= "\n\t";
-        }
+	$callpack //= '';
 qq|
 
 $pref.pm: $src $core/Types.pm
-	$depbuild\$(PERLRUNINST) \"-MPDL::PP qw[$mod $mod $pref]\" $src
+	\$(PERLRUNINST) \"-MPDL::PP qw[$mod $mod $pref $callpack]\" $src
 
 $pref.xs: $pref.pm
 	\$(TOUCH) \$@
@@ -425,17 +398,18 @@ $pref\$(OBJ_EXT): $pref.c
 	} (@_)
 }
 
-
 # This is the function to be used outside the PDL tree.
+# same format as pdlpp_postamble_int
 sub pdlpp_postamble {
-	join '',map { my($src,$pref,$mod) = @$_;
+	join '',map { my($src,$pref,$mod,$callpack) = @$_;
 	my $w = whereami_any();
 	$w =~ s%/((PDL)|(Basic))$%%;  # remove the trailing subdir
 	my $oneliner = _oneliner(qq{exit if \$ENV{DESTDIR}; use PDL::Doc; eval { PDL::Doc::add_module(q{$mod}); }});
+	$callpack //= '';
 qq|
 
 $pref.pm: $src
-	\$(PERL) "-I$w" \"-MPDL::PP qw[$mod $mod $pref]\" $src
+	\$(PERL) "-I$w" \"-MPDL::PP qw[$mod $mod $pref $callpack]\" $src
 
 $pref.xs: $pref.pm
 	\$(TOUCH) \$@
@@ -464,7 +438,7 @@ my $libsarg = $libs || $malloclib ? "$libs $malloclib " : ''; # for Win32
  return (
  	%::PDL_OPTIONS,
 	 'NAME'  	=> $mod,
-	 'VERSION_FROM' => "$w/Basic/Core/Version.pm",
+	 'VERSION_FROM' => "$w/Basic/PDL.pm",
 	 'TYPEMAPS'     => [&PDL_TYPEMAP()],
 	 'OBJECT'       => "$pref\$(OBJ_EXT)",
 	 PM 	=> {"$pref.pm" => "\$(INST_LIBDIR)/$pref.pm"},
@@ -472,7 +446,7 @@ my $libsarg = $libs || $malloclib ? "$libs $malloclib " : ''; # for Win32
 	 'INC'          => &PDL_INCLUDE()." $inc $mallocinc",
 	 'LIBS'         => $libsarg ? [$libsarg] : [],
 	 'clean'        => {'FILES'  => "$pref.xs $pref.pm $pref\$(OBJ_EXT) $pref.c"},
-     (eval ($ExtUtils::MakeMaker::VERSION) >= 6.57_02 ? ('NO_MYMETA' => 1) : ()),
+	 NO_MYMETA => 1,
  );
 }
 
@@ -483,6 +457,7 @@ sub pdlpp_stdargs {
  return (
  	%::PDL_OPTIONS,
 	 'NAME'  	=> $mod,
+	 'VERSION_FROM' => $src,
 	 'TYPEMAPS'     => [&PDL_INST_TYPEMAP()],
 	 'OBJECT'       => "$pref\$(OBJ_EXT)",
 	 PM 	=> {"$pref.pm" => "\$(INST_LIBDIR)/$pref.pm"},
@@ -491,7 +466,7 @@ sub pdlpp_stdargs {
 	 'LIBS'         => $libs ? ["$libs "] : [],
 	 'clean'        => {'FILES'  => "$pref.xs $pref.pm $pref\$(OBJ_EXT) $pref.c"},
 	 'dist'         => {'PREOP'  => '$(PERL) "-I$(INST_ARCHLIB)" "-I$(INST_LIB)" -MPDL::Core::Dev -e pdlpp_mkgen $(DISTVNAME)' },
-     (eval ($ExtUtils::MakeMaker::VERSION) >= 6.57_02 ? ('NO_MYMETA' => 1) : ()),
+	 NO_MYMETA => 1,
  );
 }
 
@@ -689,13 +664,7 @@ sub trylink {
   my $HIDE = !$hide ? '' : '>/dev/null 2>&1';
   if($^O =~ /mswin32/i) {$HIDE = '>NUL 2>&1'}
 
-  my $tempd;
-
-  $tempd = File::Temp::tempdir(CLEANUP=>1) || die "trylink: could not make TEMPDIR";
-  ### if($^O =~ /MSWin32/i) {$tempd = File::Spec->tmpdir()}
-  ### else {
-  ###    $tempd = $PDL::Config{TEMPDIR} ||
-  ### }
+  my $tempd = File::Temp::tempdir(CLEANUP=>1) || die "trylink: could not make temp dir";
 
   my ($tc,$te) = map {&$cfile($tempd,"testfile$_")} ('.c','');
   open FILE,">$tc" or die "trylink: couldn't open testfile `$tc' for writing, $!";
@@ -738,17 +707,12 @@ prints on C<STDOUT> XS text for F<Core.xs>.
 
 sub datatypes_switch {
   loadmod_Types();
-  my $ntypes = $#PDL::Types::names;
-  my @m;
-  foreach my $i ( 0 .. $ntypes ) {
-    my $type = PDL::Type->new( $i );
+  foreach my $type ( PDL::Types::types() ) {
     my $typesym = $type->symbol;
     my $typeppsym = $type->ppsym;
-    my $cname = $type->ctype;
-    $cname =~ s/^PDL_//;
-    push @m, "\tcase $typesym: retval.type = $typesym; retval.value.$typeppsym = PDL.bvals.$cname; break;";
+    my $cname = $type->shortctype;
+    print "\tcase $typesym: retval.type = $typesym; retval.value.$typeppsym = PDL.bvals.$cname; break;\n";
   }
-  print map "$_\n", @m;
 }
 
 =head2 generate_core_flags
@@ -771,12 +735,11 @@ my %flags = (
     dimschgd => { FLAG => "PARENTDIMSCHANGED" },
     tracedebug => { FLAG => "TRACEDEBUG", set => 1},
 );
-#if ( $bvalflag ) { $flags{baddata} = { set => 1, FLAG => "BADVAL" }; }
 
 sub generate_core_flags {
     # access (read, if set is true then write as well; if postset true then
     #         read first and write new value after that)
-    # to piddle's state
+    # to ndarray's state
     foreach my $name ( sort keys %flags ) {
         my $flag = "PDL_" . ($flags{$name}{FLAG} || uc($name));
         if ( $flags{$name}{set} ) {
@@ -833,21 +796,10 @@ prints on C<STDOUT> XS text with badval initialisation, for F<Core.xs>.
 sub generate_badval_init {
   loadmod_Types();
   for my $type (PDL::Types::types()) {
-    my $typename = $type->ctype;
-    $typename =~ s/^PDL_//;
+    my $typename = $type->shortctype;
     my $bval = $type->defbval;
-    if ($PDL::Config{BADVAL_USENAN} && $type->usenan) {
-      # note: no defaults if usenan
-      print "\tPDL.bvals.$typename = PDL.NaN_$type;\n"; #Core NaN value
-    } else {
-      print "\tPDL.bvals.$typename = PDL.bvals.default_$typename = $bval;\n";
-    }
+    print "\tPDL.bvals.$typename = PDL.bvals.default_$typename = $bval;\n";
   }
-#    PDL.bvals.Byte   = PDL.bvals.default_Byte   = UCHAR_MAX;
-#    PDL.bvals.Short  = PDL.bvals.default_Short  = SHRT_MIN;
-#    PDL.bvals.Ushort = PDL.bvals.default_Ushort = USHRT_MAX;
-#    PDL.bvals.Long   = PDL.bvals.default_Long   = INT_MIN;
-
 }
 
 =head2 got_complex_version
@@ -870,7 +822,7 @@ sub got_complex_version {
     $got_complex_cache{$name} = Devel::CheckLib::check_lib(
         lib => 'm',
         header => 'complex.h',
-        function => sprintf('double num; num = creal(c%s(%s));', $name, $args),
+        function => sprintf('double num; num = creal(c%s(%s)); return 0;', $name, $args),
     );
 }
 
