@@ -1,3 +1,8 @@
+package PDL;
+
+use strict;
+use warnings;
+
 =head1 NAME
 
 PDL - the Perl Data Language
@@ -37,9 +42,9 @@ For example:
  pdl> apropos xval
  xlinvals        X axis values between endpoints (see xvals).
  xlogvals        X axis values logarithmicly spaced...
- xvals           Fills a piddle with X index values...
- yvals           Fills a piddle with Y index values. See the CAVEAT for xvals.
- zvals           Fills a piddle with Z index values. See the CAVEAT for xvals.
+ xvals           Fills an ndarray with X index values...
+ yvals           Fills an ndarray with Y index values. See the CAVEAT for xvals.
+ zvals           Fills an ndarray with Z index values. See the CAVEAT for xvals.
 
 To learn more about the PDL shell, see L<perldl> or L<pdl2>.
 
@@ -81,6 +86,12 @@ cannot find what you are looking for, try here.
 
 =back
 
+=head1 DATA TYPES
+
+PDL comes with support for most native numeric data types available in C.
+2.027 added support for C99 complex numbers.  See L<PDL::Complex>,
+L<PDL::Core>, L<PDL::Ops> and L<PDL::Math> for details on usage and
+behaviour.
 
 =head1 MODULES
 
@@ -136,7 +147,6 @@ start-up modules.
    use PDL::Bad;
    use PDL::MatrixOps;
    use PDL::Math;
-   use PDL::Version;
    use PDL::IO::Misc;
    use PDL::IO::FITS;
    use PDL::IO::Pic;
@@ -145,21 +155,17 @@ start-up modules.
 
 =cut
 
-
 # set the version:
-$PDL::VERSION = '2.028_01';
+our $VERSION = '2.040_01';
 
 # Main loader of standard PDL package
 
-sub PDL::import {
-
-my $pkg = (caller())[0];
-eval <<"EOD";
-
+sub import {
+  my $pkg = (caller())[0];
+  eval <<"EOD";
 package $pkg;
 
 # Load the fundamental packages
-
 use PDL::Core;
 use PDL::Ops;
 use PDL::Primitive;
@@ -171,31 +177,18 @@ use PDL::Math;
 use PDL::MatrixOps;
 use PDL::Lvalue;
 
-# Load these for TPJ compatibility
-
+# for TPJ compatibility
 use PDL::IO::Misc;          # Misc IO (Ascii)
 use PDL::IO::FITS;          # FITS IO (rfits/wfits; used by rpic/wpic too)
 use PDL::IO::Pic;           # rpic/wpic
 
-# Load this so config/install info is available
-
-use PDL::Config;
-
-# Load this to avoid mysterious Storable segfaults
-
-use PDL::IO::Storable;
+use PDL::Config; # so config/install info is available
+use PDL::IO::Storable; # to avoid mysterious Storable segfaults
 
 EOD
-
-die $@ if $@;
-
+  die $@ if $@;
 }
 
-
-# Dummy Package PDL Statement. This is only needed so CPAN
-# properly recognizes the PDL package.
-package PDL;
-
 # support: use Inline with => 'PDL';
 # Returns a hash containing parameters accepted by recent versions of
 # Inline, to tweak compilation.  Not normally called by anyone but
@@ -211,20 +204,17 @@ sub Inline {
 ##################################################
 # Rudimentary handling for multiple Perl threads #
 ##################################################
-my $clone_skip_should_be_quiet = 0;
+our $no_clone_skip_warning = 0;
 sub CLONE_SKIP {
-    warn("* If you need to share PDL data across threads, use memory mapped data, or\n"
-		. "* check out PDL::Parallel::threads, available on CPAN.\n"
-        . "* You can silence this warning by saying `PDL::no_clone_skip_warning;'\n"
-        . "* before you create your first thread.\n")
-        unless $clone_skip_should_be_quiet;
-    PDL::no_clone_skip_warning();
-    # Whether we warned or not, always return 1 to tell Perl not to clone PDL data
+    warn <<'EOF' if !$no_clone_skip_warning;
+* If you need to share PDL data across threads, use memory mapped data, or
+* check out PDL::Parallel::threads, available on CPAN.
+* You can silence this warning by saying `$PDL::no_clone_skip_warning = 1;'
+* before you create your first thread.
+EOF
+    $no_clone_skip_warning = 1;
+    # always return 1 to tell Perl not to clone PDL data
     return 1;
 }
-sub no_clone_skip_warning {
-    $clone_skip_should_be_quiet = 1;
-}
 
-# Exit with OK status
 1;
