@@ -24,11 +24,11 @@ PDL::Graphics::LUT contains a number of colour look-up tables
 (in rgb format) and intensity ramps, and provides routines to 
 access this data.
 The format of the data is suitable for use by
-L<PDL::Graphics::PGPLOT/ctab>.
+L<ctab|PDL::Graphics::PGPLOT::Window/ctab>.
 
 Unlike the initial release of the package, the data tables are
 now stored within the PDL distribution as FITS files
-(see L<$tabledir|/$tabledir> and L<$rampdir|/$rampdir>),
+(see L</$tabledir> and L</$rampdir>),
 rather than in the module itself.
 Changes to these directories will be picked up on the next call
 to one of the package functions.
@@ -63,7 +63,7 @@ Load in the requested colour table and intensity ramp.
 
 =for usage
 
- ( $l, $r, $g, $b ) = lut_data( $table, [ $reverse, [ $ramp ] ] );
+ my ( $l, $r, $g, $b ) = lut_data( $table, [ $reverse, [ $ramp ] ] );
 
 Returns the levels and r, g, b components of the colour table
 C<$table>. If C<$reverse> is 1 (defaults to B<0> 
@@ -73,7 +73,7 @@ returned.
 If not supplied, C<$ramp> defaults to B<"ramp"> 
 (this is a linear intensity ramp).
 
-The returned values are piddles containing values in the range
+The returned values are ndarrays containing values in the range
 0 to 1 inclusive, and are floats.
 
 =head1 VARIABLES
@@ -128,6 +128,8 @@ use Exporter;
 use File::Spec;
 use File::Basename;
 
+use autodie;
+
 use PDL::Core qw/:Func :Internal/;    # Grab the Core names
 use PDL::Basic;
 use PDL::Types;
@@ -168,26 +170,21 @@ BEGIN {
 
 ############################################################################
 
+sub _lsdir_basename {
+    my ($dir, $suffix) = @_;
+    opendir my $fh, $dir;
+    map basename($_, $suffix), grep /\Q$suffix\E\z/, readdir $fh;
+}
+
 # exported functions
 
 # Return the list of available tables
-#
-sub lut_names () { 
-    my $glob = File::Spec->catfile( $tabledir, "*${suffix}" );
-    # note: really should protect any "."'s in $suffix, but too lazy
-    return map { basename($_,$suffix); } glob( $glob );
-}
+sub lut_names () { _lsdir_basename $tabledir, $suffix }
 
 # Return the list of available ramps
-#
-sub lut_ramps () { 
-    my $glob = File::Spec->catfile( $rampdir, "*${suffix}" );
-    # note: really should protect any "."'s in $suffix, but too lazy
-    return map { basename($_,$suffix); } glob( $glob );
-}
+sub lut_ramps () { _lsdir_basename $rampdir, $suffix }
 
 # Return the requested colour table 
-#
 sub lut_data ($;$$) {
     my $table   = shift;
     my $reverse = $#_ != -1 ? shift : 0;
