@@ -102,7 +102,7 @@ sub _wfits_table ($$$);
 
 =for ref
 
-Simple piddle FITS reader.
+Simple ndarray FITS reader.
 
 =for example
 
@@ -186,7 +186,7 @@ and get a large boost in speed.
 FITS image headers are stored in the output PDL and can be retrieved
 with L<hdr|PDL::Core/hdr> or L<gethdr|PDL::Core/gethdr>.  The
 L<hdrcpy|PDL::Core/hdrcpy> flag of the PDL is set so that the header
-is copied to derived piddles by default.  (This is inefficient if you
+is copied to derived ndarrays by default.  (This is inefficient if you
 are planning to do lots of small operations on the data; clear
 the flag with "->hcpy(0)" or via the options hash if that's the case.)
 
@@ -299,7 +299,7 @@ in place for adding other compression schemes.
 =head3 BAD VALUE HANDLING
 
 If a FITS file contains the C<BLANK> keyword (and has C<BITPIX E<gt> 0>), 
-the piddle will have its bad flag set, and those elements which equal the
+the ndarray will have its bad flag set, and those elements which equal the
 C<BLANK> value will be set bad.  For C<BITPIX E<lt> 0>, any NaN's are
 converted to bad (if necessary).
 
@@ -620,7 +620,7 @@ sub _rfits_image($$$$) {
   my $opt = shift;  # $opt contains the option hash
   my $pdl = shift;  # $pdl contains a pre-blessed virgin PDL
 
-  # Setup piddle structure
+  # Setup ndarray structure
   
   if(  defined($type_table->{0 + $foo->{"BITPIX"}})  ) {
       $pdl->set_datatype( $type_table->{$foo->{"BITPIX"}} );
@@ -681,30 +681,27 @@ sub treat_bscale($$){
 
     print "treating bscale...\n" if($PDL::debug);
 
-    if ( $PDL::Bad::Status ) {
-      # do we have bad values? - needs to be done before BSCALE/BZERO
-      # (at least for integers)
-      #
-      if ( $$foo{BITPIX} > 0 and exists $$foo{BLANK} ) {
-	# integer, so bad value == BLANK keyword
-	my $blank = $foo->{BLANK};
-	# do we have to do any conversion?
-	if ( $blank == $pdl->badvalue() ) {
-	  $pdl->badflag(1);
-	} else {
-	  # we change all BLANK values to the current bad value
-	  # (would not be needed with a per-piddle bad value)
-	  $pdl->inplace->setvaltobad( $blank );
-	}
-      } elsif ( $foo->{BITPIX} < 0 ) {
-	# bad values are stored as NaN's in FITS
-	# let setnanbad decide if we need to change anything
-	$pdl->inplace->setnantobad();
+    # do we have bad values? - needs to be done before BSCALE/BZERO
+    # (at least for integers)
+    if ( $$foo{BITPIX} > 0 and exists $$foo{BLANK} ) {
+      # integer, so bad value == BLANK keyword
+      my $blank = $foo->{BLANK};
+      # do we have to do any conversion?
+      if ( $blank == $pdl->badvalue() ) {
+        $pdl->badflag(1);
+      } else {
+        # we change all BLANK values to the current bad value
+        # (would not be needed with a per-ndarray bad value)
+        $pdl->inplace->setvaltobad( $blank );
       }
-      print "FITS file may contain bad values.\n"
-	if $pdl->badflag() and $PDL::verbose;
-    } # if: PDL::Bad::Status
-    
+    } elsif ( $foo->{BITPIX} < 0 ) {
+      # bad values are stored as NaN's in FITS
+      # let setnanbad decide if we need to change anything
+      $pdl->inplace->setnantobad();
+    }
+    print "FITS file may contain bad values.\n"
+      if $pdl->badflag() and $PDL::verbose;
+
     my ($bscale, $bzero);
     $bscale = $$foo{"BSCALE"}; $bzero = $$foo{"BZERO"};
     print "BSCALE = $bscale &&  BZERO = $bzero\n" if $PDL::verbose;
@@ -1758,7 +1755,7 @@ ASCII tables are not yet handled but should be.
 
 Binary tables currently only handle one vector (up to 1-D array) 
 per table entry; the standard allows more, and should be fully implemented.
-This means that PDL::Complex piddles currently can not be written to
+This means that PDL::Complex ndarrays currently can not be written to
 disk.
 
 Handling multidim arrays implies that perl multidim lists should also be
@@ -2070,7 +2067,7 @@ sub PDL::wfits {
 	      
 	      if ( $pdl->badflag() ) {
 		  if ( $BITPIX > 0 ) { my $x = &$convert(pdl(0.0));
-				       $h->{BLANK} = $x->badvalue(); }
+				       $h->{BLANK} = $x->badvalue->sclr; }
 		  else               { delete $h->{BLANK}; }
 	      }
 	      
@@ -2183,7 +2180,7 @@ sub PDL::wfits {
 	      #                    (make sure it's for the correct type)
 	      #   otherwise      - make sure the BLANK keyword is removed
 	      if ( $pdl->badflag() ) {
-		  if ( $BITPIX > 0 ) { my $x = &$convert(pdl(0.0)); $hdr{BLANK} = $x->badvalue(); }
+		  if ( $BITPIX > 0 ) { my $x = &$convert(pdl(0.0)); $hdr{BLANK} = $x->badvalue->sclr; }
 		  else               { delete $hdr{BLANK}; }
 	      }
 	      
